package tw.org.cmex.flex.timer
{
   import cmex.AppStates;
   import cmex.ExerciseCanvas;
   import flash.display.CapsStyle;
   import flash.display.JointStyle;
   import flash.display.LineScaleMode;
   import flash.events.TimerEvent;
   import flash.utils.Timer;
   import mx.core.UIComponent;
   import tw.org.cmex.flex.gd.TrajectionLine;
   
   public class StrokeOrderDemo
   {
      
      public static const EVENT_SINGLE_STROKE_FINISH:String = "SINGLE_STROKE_FINISH";
       
      
      private var _isPaused:Boolean = false;
      
      private var animator:Timer = null;
      
      private var container:ExerciseCanvas = null;
      
      private var provider:DemoStrokeUnitProvider = null;
      
      private var currDemoUnit:DemoStrokeUnit = null;
      
      private var originDelay:Number;
      
      private var demoThumbSize:Number;
      
      private var targetThumbSize:Number;
      
      public var interDuration:Number = 1000;
      
      public function StrokeOrderDemo(param1:ExerciseCanvas, param2:Number)
      {
         super();
         this.animator = new Timer(param2);
         this.demoThumbSize = AppStates.PEN_SIZE;
         this.targetThumbSize = AppStates.PEN_SIZE;
         this.originDelay = param2;
         this.container = param1;
         this.provider = new DemoStrokeUnitProvider(AppStates.getInstance().strokeList);
         this.animator.addEventListener(TimerEvent.TIMER,this.drawNext);
      }
      
      private function reset() : void
      {
         if(this.isRunning())
         {
            this.stop();
         }
         AppStates.getInstance().isDemoing = false;
         AppStates.getInstance().isStartPauseDemo = false;
         this._isPaused = false;
         this.demoThumbSize = AppStates.PEN_SIZE;
         this.targetThumbSize = AppStates.PEN_SIZE;
         this.currDemoUnit = null;
         this.provider.reset();
      }
      
      public function changeSpeed(param1:Number) : void
      {
         this.originDelay = param1;
         this.animator.delay = param1;
      }
      
      public function isRunning() : Boolean
      {
         return this.animator.running;
      }
      
      public function addEventListener(param1:String, param2:Function) : void
      {
         this.animator.addEventListener(param1,param2);
      }
      
      public function pause() : void
      {
         var _loc1_:UIComponent = null;
         if(this._isPaused)
         {
            return;
         }
         if(this.currDemoUnit != null)
         {
            if(!this.currDemoUnit.hasNext())
            {
               this.currDemoUnit = null;
            }
            else
            {
               _loc1_ = this.currDemoUnit.demoArea.getChildByName("temp") as UIComponent;
               if(_loc1_)
               {
                  this.currDemoUnit.reset();
                  _loc1_.graphics.clear();
               }
            }
         }
         this._isPaused = true;
         this.animator.stop();
      }
      
      public function isPaused() : Boolean
      {
         return this._isPaused;
      }
      
      public function replay() : void
      {
         this.stop();
         this.container.clean();
         this.start();
      }
      
      public function stop() : void
      {
         this.animator.stop();
         this.animator.dispatchEvent(new TimerEvent(CmexTimer.EVENT_TIMER_STOPPED));
         this._isPaused = false;
         this.reset();
      }
      
      public function start() : void
      {
         AppStates.getInstance().isDemoing = true;
         this._isPaused = false;
         this.animator.start();
         this.animator.delay = this.originDelay;
      }
      
      private function drawNext(param1:TimerEvent) : void
      {
         var _loc4_:Array = null;
         var _loc5_:int = 0;
         var _loc6_:UIComponent = null;
         var _loc7_:UIComponent = null;
         if(this.currDemoUnit == null || !this.currDemoUnit.hasNext())
         {
            if(this.provider.hasNext())
            {
               this.currDemoUnit = this.provider.next();
               this.animator.delay = this.originDelay;
            }
            else
            {
               this.stop();
               return;
            }
         }
         if(this.currDemoUnit.isFirst())
         {
            _loc4_ = new Array();
            _loc5_ = 0;
            while(_loc5_ < this.container.strokeArea.numChildren)
            {
               _loc7_ = this.container.strokeArea.getChildAt(_loc5_) as UIComponent;
               if(_loc7_.name == "temp" || _loc7_.name == "mask")
               {
                  _loc4_.push(_loc7_);
               }
               _loc5_++;
            }
            for each(_loc6_ in _loc4_)
            {
               this.container.strokeArea.removeChild(_loc6_);
            }
            this.currDemoUnit.maskArea.name = "mask";
            this.currDemoUnit.demoArea.name = "temp";
            this.container.strokeArea.addChild(this.currDemoUnit.maskArea);
            this.container.strokeArea.addChild(this.currDemoUnit.demoArea);
            this.currDemoUnit.demoArea.mask = this.currDemoUnit.maskArea;
         }
         var _loc2_:TrajectionLine = this.currDemoUnit.nextLine();
         this.targetThumbSize = AppStates.PEN_SIZE;
         this.demoThumbSize = AppStates.PEN_SIZE;
         var _loc3_:UIComponent = new UIComponent();
         _loc3_.name = "temp";
         _loc3_.graphics.lineStyle(this.demoThumbSize,0,1,false,LineScaleMode.NORMAL,CapsStyle.ROUND,JointStyle.ROUND,20);
         _loc2_.apply(_loc3_.graphics);
         this.currDemoUnit.demoArea.addChild(_loc3_);
         if(!this.currDemoUnit.hasNext())
         {
            this.container.strokeArea.addChild(this.currDemoUnit.outline);
         }
         if(!this.currDemoUnit.hasNext() && this.provider.hasNext())
         {
            if(AppStates.getInstance().isStartPauseDemo)
            {
               this.pause();
               this.animator.dispatchEvent(new TimerEvent(StrokeOrderDemo.EVENT_SINGLE_STROKE_FINISH));
            }
            else
            {
               this.animator.delay = this.interDuration;
            }
         }
      }
   }
}
